/*************************************************************************

    File: eo-dict-sem.pl
    Copyright (C) 2008 Johannes Dellert

    This file is part of LARA, version 1.0 (August 2008).

*************************************************************************/

/*========================================================================
   Semantic Lexicon for Esperanto
========================================================================*/

% Complex lexical rule describing the different interpretations for nouns

semLex(sub,Lemma,Sem):-
   % Standard Noun
   (
    Sem = lam(X,Formula),
    compose(Formula,Lemma,[X])
   );
   % Proper Nouns (syntactic parser can't distinguish them from nouns!)
   (
    Sem = lam(P,app(P,Lemma))
   ), !.


% Copula

semLex(vrb, esti, Sem) :-
  Sem = lam(K,lam(Y,app(K,lam(X,eq(Y,X))))), !.


% Complex lexical rule for verbs that can either be transitive or intransitive

semLex(vrb,Lemma,Sem):-
  % Intransitive Verb
  (
    Sem = lam(X,Formula),
    compose(Formula,Lemma,[X])
  );
  % Transitive Verb
  (
    Sem = lam(K,lam(Y,app(K,lam(X,Formula)))),
    compose(Formula,Lemma,[Y,X])
  ), !.


% Negation

semLex(adv,ne,Sem) :-
  Sem = lam(P,lam(X,not(app(P,X))));
  Sem = lam(P,not(P)), !.

% Universal Quantification

semLex(prn, cxiu, Sem) :-
  Sem = lam(U,lam(V,all(X,imp(app(U,X),app(V,X))))), !.

% Existential Quantification (usually implicit)

semLex(prn, iu, Sem) :-
  Sem = lam(U,lam(V,some(X,and(app(U,X),app(V,X))))), !.

% Relative / Interrogative pronoun "kiu"

semLex(prn, kiu , Sem):-
   Sem = lam(P,lam(Q,lam(X,and(app(P,X),app(Q,X)))));
   Sem = lam(Q,que(X,X,app(Q,X))), !.

%  Relative / Interrogative pronoun "kio"

semLex(prn, kio , Sem):-
   Sem = lam(P,lam(Q,lam(X,and(app(P,X),app(Q,X))))),
   Sem = lam(Q,que(X,X,app(Q,X))), !. 

% Other pronouns simply treated as constants

semLex(prn, Lemma, Sem) :-
  Sem = lam(P,app(P,Lemma)), !.

% Adjectives

semLex(adj, Lemma, Sem) :-
  Sem = lam(P,lam(X,and(F,app(P,X)))),
  compose(F,Lemma,[X]), !.

% Prepositions

semLex(prp, Lemma, Sem) :-
   Sem = lam(K,lam(P,lam(Y,and(app(K,lam(X,F)),app(P,Y))))),
   compose(F,Lemma,[Y,X]), !.

% Verbal coordination

semLex(konv, kaj, Sem) :-
  Sem = lam(X,lam(Y,lam(P,lam(Q,and(app(app(X,Q),P),app(app(Y,Q),P))))));
  Sem = lam(X,lam(Y,lam(P,and(app(X,P),app(Y,P))))), !.

semLex(konv, aux, Sem) :-
  Sem = lam(X,lam(Y,lam(P,lam(Q,or(app(app(X,Q),P),app(app(Y,Q),P))))));
  Sem = lam(X,lam(Y,lam(P,or(app(X,P),app(Y,P))))), !.

% Subordinating conjunctions

semLex(sbk, se, Sem) :-
  Sem = lam(X,lam(Y,imp(X,Y))), !.

% default for uninterpretable tokens: unknown semantics
% is processed even if there are viable alternatives if these don't end with a cut! 

semLex(_,_,unknown).


/*========================================================================
   Directionality Constraints for Esperanto

    -  specify in which direction heads and dependents are combined
    -  enriched by additional instructions for implicit quantification
========================================================================*/

directional(vrb:A,  subj,   sub:B,  app(B,A)).
directional(vrb:A,  obj,    sub:B,  app(A,B)).
directional(vrb:A,  pred,   sub:B,  app(A,B)).
directional(konv:A, subj,   sub:B,  app(B,A)).
directional(konv:A,  obj,    sub:B,  app(B,A)).

% implicit existential quantification
directional(vrb:A,  subj,   sub:B,  
  app(app(lam(U,lam(V,some(X,and(app(U,X),app(V,X))))),B),A)) :- !.
directional(vrb:A,  obj,    sub:B,  
  app(A,app(lam(U,lam(V,some(X,and(app(U,X),app(V,X))))),B))) :- !.
directional(vrb:A,  pred,   sub:B,  
  app(A,app(lam(U,lam(V,some(X,and(app(U,X),app(V,X))))),B))) :- !.
directional(konv:A, subj,   sub:B,  
  app(app(lam(U,lam(V,some(X,and(app(U,X),app(V,X))))),B),A)) :- !.

directional(vrb:A,  proa,   prp:B,  app(B,A)) :- !.
directional(vrb:A,  proa,   sbk:B,  app(B,A)) :- !.
directional(vrb:A,  moda,   adv:B,  app(B,A)) :- !.
directional(sub:A,  det,    prn:B,  app(B,A)) :- !.
directional(sub:A,  atr1,   adj:B,  app(B,A)) :- !.
directional(sub:A,  atr1,   prn:B,  app(B,A)) :- !.
directional(sub:A,  rel,    vrb:B,  app(B,A)) :- !.
directional(prp:A,  parg,   sub:B,  app(A,B)) :- !.
directional(sbk:A,  subc,   vrb:B,  app(A,B)) :- !.

directional(_:A,    _,      _:B,    app(B,A)).
directional(_:A,    _,      _:B,    app(A,B)).

/*========================================================================
   Precedence Constraints for Esperanto

    - specify in which order dependents are combined with their head
========================================================================*/

precedence(vrb,   obj,    subj).
precedence(vrb,   pred,   subj).
precedence(vrb,   subj,   proa).
precedence(sub,   atr1,   det).
precedence(konv,  konv1,  konv2).
precedence(konv,  konv2,  obj).
precedence(konv,  obj,    subj).
